package com.pj.face.config;

import cn.hutool.core.io.FileUtil;
import cn.hutool.log.StaticLog;
import com.arcsoft.face.ActiveFileInfo;
import com.arcsoft.face.EngineConfiguration;
import com.arcsoft.face.FaceEngine;
import com.arcsoft.face.FunctionConfiguration;
import com.arcsoft.face.enums.DetectMode;
import com.arcsoft.face.enums.DetectOrient;
import com.arcsoft.face.enums.ErrorInfo;
import com.pj.current.config.SystemObject;
import lombok.extern.slf4j.Slf4j;

import java.io.File;

@Slf4j
public class FaceEngineConfig {

    public static FaceEngine faceEngine = null;

    public static void initFaceEngine() {
        String sdk = SystemObject.config.getFaceLibs();
        log.info("exit:{}", FileUtil.exist(sdk + File.separator + "libarcsoft_face.dll"));
        faceEngine = new FaceEngine(sdk);
        //激活引擎
        int errorCode = faceEngine.activeOnline(SystemObject.config.getArcsoftAppid(), SystemObject.config.getArcsoftSdkKey());
        log.info("activate:{}", errorCode);
        StaticLog.info("activate:{}", errorCode);

        if (errorCode != ErrorInfo.MOK.getValue() && errorCode != ErrorInfo.MERR_ASF_ALREADY_ACTIVATED.getValue()) {
            StaticLog.error("引擎激活失败");
        }


        ActiveFileInfo activeFileInfo = new ActiveFileInfo();
        errorCode = faceEngine.getActiveFileInfo(activeFileInfo);
        if (errorCode != ErrorInfo.MOK.getValue() && errorCode != ErrorInfo.MERR_ASF_ALREADY_ACTIVATED.getValue()) {
            StaticLog.error("获取激活文件信息失败");
        }

        //引擎配置
        EngineConfiguration engineConfiguration = new EngineConfiguration();
        engineConfiguration.setDetectMode(DetectMode.ASF_DETECT_MODE_IMAGE);
        engineConfiguration.setDetectFaceOrientPriority(DetectOrient.ASF_OP_ALL_OUT);
        engineConfiguration.setDetectFaceMaxNum(10);
        engineConfiguration.setDetectFaceScaleVal(16);
        //功能配置
        FunctionConfiguration functionConfiguration = new FunctionConfiguration();
        functionConfiguration.setSupportFaceDetect(true);
        functionConfiguration.setSupportFaceRecognition(true);
        functionConfiguration.setSupportLiveness(true);
        engineConfiguration.setFunctionConfiguration(functionConfiguration);


        //初始化引擎
        errorCode = faceEngine.init(engineConfiguration);

        if (errorCode != ErrorInfo.MOK.getValue()) {
            StaticLog.error("初始化引擎失败");
        }


    }
}
