package com.pj.face.handler.impl;

import cn.hutool.core.io.FileUtil;
import com.arcsoft.face.FaceFeature;
import com.arcsoft.face.FaceInfo;
import com.arcsoft.face.FaceSimilar;
import com.arcsoft.face.LivenessInfo;
import com.arcsoft.face.toolkit.ImageInfo;
import com.pj.current.config.SystemObject;
import com.pj.face.brand.FaceBrand;
import com.pj.face.config.FaceEngineConfig;
import com.pj.face.handler.IFaceHandler;
import com.pj.face.handler.VerifyVO;
import com.pj.utils.sg.AjaxError;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

import static com.arcsoft.face.toolkit.ImageFactory.getRGBData;

/**
 * 基于ArcSoft Face的人��识别
 */
@Service
@Slf4j
public class ArcsoftFaceHandlerImpl implements IFaceHandler {
    @Override
    public boolean checkIsLive(String baseImg) {
        List<LivenessInfo> livenessInfoList = new ArrayList<>();
        FaceEngineConfig.faceEngine.getLiveness(livenessInfoList);
        return livenessInfoList.get(0).getLiveness() == 1;
    }

    @Override
    public void createPersonByBase(String faceId, String name, String imageBase) {
        byte[] imageBytes = Base64.getDecoder().decode(imageBase);
        String facePath = SystemObject.getFaceImgPath();
        File outputFile = new File(facePath + File.separator + faceId + ".png");
        FileUtil.del(outputFile);
        // 创建输出流
        try (FileOutputStream fileOutputStream = new FileOutputStream(outputFile)) {
            // 写入数据到文件
            fileOutputStream.write(imageBytes);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void deletePerson(String faceId) {
        String facePath = SystemObject.getFaceImgPath();
        File outputFile = new File(facePath + File.separator + faceId + ".png");
        FileUtil.del(outputFile);
    }

    @Override
    public boolean checkHasPerson(String img) {
        byte[] imageBytes = Base64.getDecoder().decode(img);
        ImageInfo imageInfo = getRGBData(imageBytes);
        List<FaceInfo> faceInfoList = new ArrayList<>();
        FaceEngineConfig.faceEngine.detectFaces(imageInfo.getImageData(), imageInfo.getWidth(), imageInfo.getHeight(), imageInfo.getImageFormat(), faceInfoList);
        return faceInfoList.size() == 1;
    }

    @Override
    public VerifyVO verify(String faceId, String baseImage) {
        VerifyVO vo = new VerifyVO();
        String facePath = SystemObject.getFaceImgPath();
        File file = new File(facePath + File.separator + faceId + ".png");
        if (!file.exists()){
            vo.setMsg("人员ID不存在");
            return vo;
        }
        ImageInfo imageInfo2 = getRGBData(file);
        byte[] imageBytes = Base64.getDecoder().decode(baseImage);
        ImageInfo imageInfo = getRGBData(imageBytes);
        List<FaceInfo> faceInfoList = new ArrayList<>();
        FaceEngineConfig.faceEngine.detectFaces(imageInfo.getImageData(), imageInfo.getWidth(), imageInfo.getHeight(), imageInfo.getImageFormat(), faceInfoList);
        if (faceInfoList.size() == 0) {
            vo.setMsg("No faces");
            return vo;
            //throw new AjaxError("请正视摄像头");
        }
        if (faceInfoList.size() > 1) {
            vo.setMsg("图片中存在多个人脸");
            return vo;
            //throw new AjaxError("图片中存在多个人脸");
        }
        FaceInfo faceInfo = faceInfoList.get(0);
        FaceFeature faceFeature = new FaceFeature();
        FaceEngineConfig.faceEngine.extractFaceFeature(imageInfo.getImageData(), imageInfo.getWidth(), imageInfo.getHeight(), imageInfo.getImageFormat(), faceInfo, faceFeature);
        System.out.println("特征值大小：" + faceFeature.getFeatureData().length);


        //人脸检测2

        List<FaceInfo> faceInfoList2 = new ArrayList<FaceInfo>();
        FaceEngineConfig.faceEngine.detectFaces(imageInfo2.getImageData(), imageInfo2.getWidth(), imageInfo2.getHeight(), imageInfo2.getImageFormat(), faceInfoList2);
        System.out.println(faceInfoList2);

        //特征提取2
        FaceFeature faceFeature2 = new FaceFeature();
        FaceEngineConfig.faceEngine.extractFaceFeature(imageInfo2.getImageData(), imageInfo2.getWidth(), imageInfo2.getHeight(), imageInfo2.getImageFormat(), faceInfoList2.get(0), faceFeature2);
        System.out.println("特征值大小：" + faceFeature2.getFeatureData().length);

        FaceFeature targetFaceFeature = new FaceFeature();
        targetFaceFeature.setFeatureData(faceFeature.getFeatureData());
        FaceFeature sourceFaceFeature = new FaceFeature();
        sourceFaceFeature.setFeatureData(faceFeature2.getFeatureData());
        FaceSimilar faceSimilar = new FaceSimilar();
        FaceEngineConfig.faceEngine.compareFaceFeature(targetFaceFeature, sourceFaceFeature, faceSimilar);
        System.out.println("相似度：" + faceSimilar.getScore());
        boolean result = faceSimilar.getScore() > 0.75f;
        String msg = result ? "人脸一致" : "不匹配";
        vo.setSuccess(result);
        vo.setMsg(msg);
        return vo;
    }

    @Override
    public List<String> faceSearch(String baseImg) {
        return null;
    }

    @Override
    public FaceBrand faceBrand() {
        return FaceBrand.ARCSOFT;
    }
}
